/*
 * Example1.java
 *
 * Umgestellt auf lokales LgDatum statt java.sql.Date
 * Created on 28. Februar 2001, 15:50
 */

package bank3tier._lgTest;

import bank3tier.lg.LgApplication;
import bank3tier.lg.LgKunde;
import bank3tier.lg.LgKunde_List;
import bank3tier.lg.LgKonto;
import bank3tier.lg.LgDatum;

/**
  Zweck: Erzeugt einige Kunden, einige von diesen verwaltete Konten
  und fhrt einige berweisungen aus.

  @author  Christoph Knabe
 */
public class Example1 extends Object {

    /** Do not create objects of me from outside*/
    private Example1() {
    }//Example1

    /**create objects, link and work with them*/
    public void work() throws Exception {
        bank3tier.db.TestAll.setUpEmptyDB();
        LgApplication.init();
        
        //Einige Kunden erfassen:
        final LgKunde mueller = new LgKunde();
        mueller.setName("Manfred Mller");
        mueller.setGeburt(LgDatum.fromString("27.02.1960"));
        mueller.erfassen();
        final LgKunde knabe = new LgKunde();
        knabe.setName("Christoph Knabe");
        knabe.setGeburt(LgDatum.fromString("31.05.1954"));
        knabe.erfassen();
        final LgKunde skoric = new LgKunde();
        skoric.setName("Gordana Skoric-Knabe");
        skoric.setGeburt(LgDatum.fromString("14.11.1962"));
        skoric.erfassen();

        //Fr diese Kunden Konten erffnen:
        final LgKonto muellerGiroKonto = mueller.kontoEroeffnen();
        final LgKonto muellerSparKonto = mueller.kontoEroeffnen();
        final LgKonto knabeKonto  = knabe.kontoEroeffnen();
        final LgKonto TSV_GuthsmuthsKonto = knabe.kontoEroeffnen();

        //Skoric als Verwalter fr knabeKonto einsetzen:
        knabeKonto.verwalterHinzufuegen(skoric);
        //Mller als Verwalter fr Konto des TSV Guthsmuths einsetzen:
        TSV_GuthsmuthsKonto.verwalterHinzufuegen(mueller);

        //Einige berweisungen ttigen:
        muellerGiroKonto.ueberweisen(muellerSparKonto, 200000, mueller);
        muellerGiroKonto.ueberweisen(TSV_GuthsmuthsKonto, 2000, mueller);
        knabeKonto.ueberweisen(muellerGiroKonto, 1998, knabe);

        //Zeige Liste aller Kunden:
        System.out.println("Liste aller Kunden");
        System.out.println("==================");
        System.out.println("oid,\tName,\tGeburt");
        final LgKunde_List kundeList = LgApplication.dummyKunde().alleSuchen();
        for(final LgKunde_List.Iterator i=kundeList.listIterator(); i.hasNext();){
          final LgKunde k = i.next();
          System.out.println(k.getOid() + ",\t" + k.getName() + ",\t" + k.getGeburt()); 
        }
        //Zeige Status und Kontostand aller von Kunde 0 verwalteten Konten:
        final LgKunde kunde0 = kundeList.get(0);
        kunde0.printKontenStaende();

        //Demonstriert eindeutiges Suchen:
        final LgDatum suchDatum = LgDatum.fromString("31.05.1954");
        System.out.println("Zeige den Kunden, der am " + suchDatum + " geboren wurde:");
        final LgKunde geb540531Kunde = LgApplication.dummyKunde().suchen(suchDatum);
        System.out.println(geb540531Kunde);
        
        //Alle diese Operationen drfen von auerhalb der Lg-Schicht aus nicht
        //zugreifbar sein und mssen daher einen Kompilationsfehler provozieren:
        //muellerGiroKonto.insert();
        //muellerGiroKonto.retrieve();
        //muellerGiroKonto.update();
        //muellerGiroKonto.delete();


    }//work

    /**Do some work using the application logic tier (Lg) */
    public static void main (final String[] i_args) throws java.sql.SQLException {
        final Example1 example = new Example1();
        try{example.work();}
        catch(Exception ex){multex.Msg.printReport(ex);  System.exit(1);}
        //DbObject.deleteTables();
    }

}//Example1