/*
 * DbBankApplication.java
 *
 * Created on 28. Mai 2001, 14:59
 */

package bank3tier.db;

/**Implements all application specific database access code for the Bank application.
 * @author  Christoph Knabe at 2001-05-27
 */
public class DbBankApplication implements DbApplication {

    /** Creates new DbBankApplication */
    public DbBankApplication() {
    }

    /**Creates an empty instance of the Bank3Tier database without opening it*/
    public void createEmptyDB(){
        final String emptyDbFileName = "EmptyDB.mdb";
		final String dbFileName = "Bank3Tier.mdb";
        try{_copyFile(emptyDbFileName, dbFileName);
        }catch(Exception ex){
            throw new multex.Failure(
            "Failure creating empty data base {1} from source file {0}",
            ex, emptyDbFileName, dbFileName
            );
        }
    }//createEmptyDB


    public void openDB(){
        //Instead of the following line you could create a JORA session
        //using another driver than JDBC-ODBC-bridge:
        _session = new jora.Session(sun.jdbc.odbc.JdbcOdbcDriver.class);
        _session.open("jdbc:odbc:Bank3Tier", "", "");
    }

    public jora.Session getSession(){return _session;}

    /**Creates the empty DB tables Kunde, Konto, Verwaltet in SQL for Access97.
    */
    public void createTables(final String i_oidSuffix)
    throws java.sql.SQLException {
	_session.execute("create table Kunde ("
            +" oid INTEGER CONSTRAINT KundeKey PRIMARY KEY"
            +",db__updateVersion INTEGER NOT NULL"
            +",name VARCHAR(50) NOT NULL"
            +",geburt DATE NOT NULL"
            +" )"
        );
	_session.execute("create table Konto ("
            +" oid INTEGER CONSTRAINT KontoKey PRIMARY KEY"
            +",db__updateVersion INTEGER NOT NULL"
            +",saldo INTEGER NOT NULL"
            +",eroeffnet DATE NOT NULL"
            +",geschlossen DATE"
            +" )"
        );
	_session.execute("create table Verwaltet ("
            +" oid INTEGER CONSTRAINT VerwaltetKey PRIMARY KEY"
            +",db__updateVersion INTEGER NOT NULL"
            +",kunde" + i_oidSuffix + " INTEGER NOT NULL"
            +",konto" + i_oidSuffix + " INTEGER NOT NULL"
            +",inhaber LOGICAL NOT NULL"
            +" )"
        );
    }//createTables

    /**Deletes the application specific tables Kunde, Konto, Verwaltet if exist.*/
    public void deleteTables(){
        _session.execute("drop table Kunde if exists");
        _session.execute("drop table Konto if exists");
        _session.execute("drop table Verwaltet if exists");
    }

    /**Copies the given in file onto the given out file in binary mode*/
    private static void _copyFile(final String i_inFileName, final String i_outFileName)
    throws  java.io.IOException
    {
        final java.io.FileInputStream  in  = new java.io.FileInputStream (i_inFileName);
        final java.io.FileOutputStream out = new java.io.FileOutputStream(i_outFileName);
        final byte[] buffer = new byte[1024];
        for(;;){
            final int len = in.read(buffer);
            if(len<=0){break;}
			out.write(buffer, 0, len);
        }//for
    }//_copyFile

    private jora.Session _session;

}