/*
 * LgKonto.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:44
 */

package bank3tier.lg;
import  bank3tier.db.DbObject;

/** * Ein Konto bei einer Bank.
 * @author  Christoph Knabe
 */
public class LgKonto extends DbObject {
    //Attribute:
    private int saldo = 0;
    private java.sql.Date eroeffnet
    = new java.sql.Date(new java.util.Date().getTime());
    private java.sql.Date geschlossen = null;

    /**Returns the actual account balance in cents*/
    public int getSaldo(){return saldo;}

    /**The minimal account balance, which the bearer of the account may not
      subcede.
    */
    public static final int mindestSaldo = -1000000;

    /** Returns the LgKonto object with the OID i_oid.
     *  @throws NotFoundExc If in i_table no object with OID i_oid was found.
     *  @see bank3tier.db.DbObject
     */
    public static LgKonto selectByOid(final Integer i_oid)
    throws DbObject.OidNotFoundExc
    {
        return (LgKonto)dbSelectByOid(table, i_oid);
    }

    /**Adds the client i_kunde as an additional manager of this account.*/
    public void verwalterHinzufuegen(final LgKunde i_kunde)throws SchonVerwalterExc{
        dbBeginTransaction();
        boolean ok = false;
        try {
            if(LgVerwaltet.dummy.dbAreLinked(i_kunde, this))
            {throw new SchonVerwalterExc(i_kunde, this);}

            final LgVerwaltet verwaltet = new LgVerwaltet(i_kunde, this);
            verwaltet.setzen(false); //kein Inhaber
            verwaltet.dbInsert();
            ok = true;
        }catch(SchonVerwalterExc ex){throw ex;
        }catch(Exception ex){
            throw new multex.Failure(
                "Verwalter konnte nicht hinzugefgt werden.", ex
            );
        }
        finally{dbEndTransaction(ok);}
    }

    /**Sends the money amount i_betrag in cents from this account to the
      destination account io_zielKonto. The operation is only allowed, if the
      sender client i_auftraggeber is an authorized manager of this source
      account.
    */
    public void ueberweisen(
        final LgKonto io_zielKonto, final int i_betrag,
        final LgKunde i_auftraggeber
    ) throws OhneRechteExc, MindestSaldoExc
    {
        dbBeginTransaction();
        boolean ok = false;
        try {
            //1. Fehlerprfung:
            if(!LgVerwaltet.dummy.dbAreLinked(i_auftraggeber, this))
            {throw new OhneRechteExc(i_auftraggeber, this);}
            final int neuSaldo = saldo - i_betrag;
            if(neuSaldo < mindestSaldo){throw new MindestSaldoExc();}
            //2. Vernderungen im Arbeitsspeicher vornehmen:
            this.saldo = neuSaldo;
            io_zielKonto.saldo += i_betrag;
            //3. Vernderungen in DB sichern:
            this.dbUpdate();
            io_zielKonto.dbUpdate();
            ok = true;
        }catch(MindestSaldoExc ex){throw ex;
        }catch(OhneRechteExc ex){throw ex;
        }catch(Exception ex){throw new multex.Failure(
            "Betrag {0} konnte nicht berwiesen werden", ex, new Long(i_betrag)
            );
        }finally{dbEndTransaction(ok);}
    }

    public class OhneRechteExc extends multex.Exc {
        public OhneRechteExc(final LgKunde i_kunde, final LgKonto i_konto){
            super("Kunde mit OID {0} darf nicht ber Konto mit OID {1} verfgen",
                i_kunde.getOid(), i_konto.getOid()
            );
        }
    }

    public class MindestSaldoExc extends multex.Exc {
        public MindestSaldoExc(){
            super("Mindestsaldo {0} Cent wrde unterschritten werden",
                new Long(LgKonto.this.mindestSaldo)
            );
        }
    }

    public class SchonVerwalterExc extends multex.Exc {
        public SchonVerwalterExc(final LgKunde i_kunde, final LgKonto i_konto){
            super("Kunde mit OID {0} ist schon Verwalter von Konto mit OID {1}",
                i_kunde.getOid(), i_konto.getOid()
            );
        }
    }

    /** Creates new LgKonto */
    /*package*/ LgKonto() {
    }

    //public void schlieen(){}

    private static final jora.Table table = dbNewTable(LgKonto.class);
    protected jora.Table dbTable(){return table;}

}