/*
 * LgKunde.java
 *
 * 2001-04-22  Knabe  extends DbObject
 * Created on 28. Februar 2001, 15:34
 */

package bank3tier.lg;
import  bank3tier.db.DbObject;
/**
 * Ein Kunde einer Bank.
 * @author  Christoph Knabe
 */
public class LgKunde extends DbObject {
    private String name;
    private java.sql.Date geburt;
    //private LgDatum geburt; //geht nicht, obwohl von java.sql.Date abgeleitet

    public void setName(final String i_name){name = i_name;}
    public String getName(){return name;}

    public void setGeburt(final LgDatum i_geburt){geburt = i_geburt.toDate();}
    public LgDatum getGeburt(){return new LgDatum(geburt);}

    /**Stores this Kunde object first time into the database
      as an external checked transaction.
    */

    public LgKunde erfassen(){
        dbBeginTransaction();
        boolean ok = false;
        try {
            dbInsert();
            ok = true;
            return this;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu erfassen", ex, name);
        }finally{dbEndTransaction(ok);}
    }

    /**ndert das Kundenobjekt als externe, geprfte Transaction*/
    public void aendern(){
        dbBeginTransaction();
        boolean ok = false;
        try {
            dbUpdate();
            ok = true;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu ndern", ex, name);
        }finally{dbEndTransaction(ok);}
    }

    /**Lscht das Kunde object as external checked transaction*/
    public void loeschen(){
        dbBeginTransaction();
        boolean ok = false;
        try {
            dbDelete();
            ok = true;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, den Kunden {0} zu lschen", ex, getName());
        }finally{dbEndTransaction(ok);}
    }

    /**Erffnet ein neues Konto, dessen Inhaber der aktuelle Kunde wird.
      As external checked transaction.
      @return das neue Konto
    */
    public LgKonto kontoEroeffnen(){
        dbBeginTransaction();
        boolean ok = false;
        try {
            final LgKonto konto = new LgKonto();
            konto.dbInsert();
            final LgVerwaltet verwaltet = new LgVerwaltet(this, konto);
            verwaltet.setzen(true); //als Inhaber
            verwaltet.dbInsert();
            ok = true;
            return konto;
        }catch(Exception ex){throw new multex.Failure(
            "Es war nicht mglich, ein Konto zu erffnen", ex);
        }finally{dbEndTransaction(ok);}
    }

    /**Sets these attributes in the actual object.
      As internal, unpersistent operation
    */
    public void setzen(final String i_name, final LgDatum i_geburt){
        name = i_name;   geburt = i_geburt.toDate();
    }


    /**Liefert alle Kunden*/
    public LgKunde_List alleSuchen(){
        dbBeginTransaction();
        boolean ok = false;
        try {
            final jora.Cursor c = table.select("");
            final LgKunde_List result = new LgKunde_List();
            result.addAll(c);
            return result;
        }catch(Exception ex){throw new multex.Failure(
            "Failure searching all Kunde-objects", ex);
        }finally{dbEndTransaction(ok);}
    }

    /**Liefert den Kunden mit dem angegebenen Geburtsdatum*/
    public /*static*/ LgKunde suchen(final LgDatum i_geburtsDatum)
    throws KeinKundeExc, DbObject.ExampleNotUniqueExc
    {
        final java.sql.Date geburtDate = i_geburtsDatum.toDate();
        dbBeginTransaction();
        boolean ok = false;
        try {
            final LgKunde example = new LgKunde();
            example.geburt = geburtDate;
            final LgKunde result = (LgKunde)example.dbSelectUniqueByExample();
            ok = true;
            return result;
        }catch(DbObject.ExampleNotFoundExc ex){throw new KeinKundeExc(geburtDate);
        }finally{dbEndTransaction(ok);}
    }

    /**Listet alle Konten des aktuellen Kunden auf mit ihrem Saldo*/
    public void printKontenStaende(){
        dbBeginTransaction();
        boolean ok = false;
        try{
            //QueryByExample: Alle Verwaltet-Links dieses Kunden:
            final LgVerwaltet query = new LgVerwaltet(this);
            final jora.Cursor c = query.table.queryByExample(query);
            System.out.println("-----------------------------");
            System.out.println("KontenStnde von Kunde " + name);
            System.out.println("-----------------------------");
            System.out.println("KontoOid,\tStatus,\tSaldo");
            for(;;){
                final LgVerwaltet verwaltet = (LgVerwaltet)c.next();
                if(verwaltet==null){break;}
                final String status = verwaltet.inhaber() ? "Inhaber" : "Verwalter";
                final LgKonto konto = (LgKonto)verwaltet.dbGetlink(LgKonto.class);
                System.out.println("#" + konto.getOid() + " " + status+ " " + konto.getSaldo());
            }
            c.close();
        }catch(Exception ex){throw new multex.Failure(
            "Fehler beim Ausgeben der KontenStnde for Kunde OID {0}", ex, getOid());
        }finally{dbEndTransaction(ok);}
    }

    public String toString(){
        return dbTable().name() + " {oid=" + getOid() + ", name=" + name
            + ", geburt=" + geburt + "}"
        ;
    }

    /**Es wurde kein Kunde mit Geburtsdatum {0,date} gefunden*/
    public static class KeinKundeExc extends multex.Exc {
        public KeinKundeExc(final java.sql.Date i_geburtDate){
            super("Es wurde kein Kunde mit Geburtsdatum {0,date} gefunden", i_geburtDate);
        }
    }

    /** Creates new empty LgKunde */
    public LgKunde() {}

    private static final jora.Table table = dbNewTable(LgKunde.class);
    protected jora.Table dbTable(){return table;}

}//LgKunde