package bank3tier.lg;

public class TestLgKunde extends junit.framework.TestCase {

    /**Erwartete Inhalte der Datenbank*/
    private static LgKunde _mueller, _knabe, _skoric;

	public TestLgKunde(final String i_name) {
		super(i_name);
	}

    public void testErfassen(){
        //3 Musterkunden erfassen:
        _mueller = _erfassen("Manfred Mller", "27.02.1960");
        _knabe = _erfassen("Christoph Knabe", "31.05.1954");
        _skoric = _erfassen("Gordana Skoric-Knabe", "14.11.1962");
        assertEquals(new Integer(2), _knabe.getOid());
        assertEquals("Christoph Knabe", _knabe.getName());
        assertEquals("31.05.1954", _knabe.getGeburt().toString());
    }

    public void testEinzelnenSuchen()
    throws LgKunde.KeinKundeExc, bank3tier.db.DbObject.ExampleNotUniqueExc
    {
        final LgKunde knabe
        = LgApplication.dummyKunde().suchen(LgDatum.fromString("31.05.1954"));
        assertEquals(_knabe.toString(), knabe.toString());

        try{final LgKunde kunde = knabe.suchen(LgDatum.fromString("01.01.1970"));
        	fail("LgKunde should not be found");
        }catch(LgKunde.KeinKundeExc ex){} //expected
    }

    public void testAlleSuchen(){
        final LgKunde_List kunden = LgApplication.dummyKunde().alleSuchen();
		assertEquals(3, kunden.size());

        final LgKunde mueller = kunden.get(0);
        assertEquals(_mueller.toString(), mueller.toString());

        final LgKunde knabe   = kunden.get(1);
        assertEquals(_knabe.toString(), knabe.toString());

        final LgKunde skoric  = kunden.get(2);
        assertEquals(_skoric.toString(), skoric.toString());
    }

    public void testAendern()
    throws LgKunde.KeinKundeExc, bank3tier.db.DbObject.ExampleNotUniqueExc
    {
        final LgKunde knabeAlt
        = LgApplication.dummyKunde().suchen(LgDatum.fromString("31.05.1954"));
        assertEquals(_knabe.toString(), knabeAlt.toString());

        knabeAlt.setName("Prof. Knabe");
        knabeAlt.setGeburt(LgDatum.fromString("31.12.2000"));
        knabeAlt.aendern();
		_knabe = knabeAlt;

        final LgKunde knabeNeu
        = LgApplication.dummyKunde().suchen(LgDatum.fromString("31.12.2000"));
        assertEquals(_knabe.toString(), knabeNeu.toString());
    }

    public void testAlleLoeschen(){
        //Alle Kunden wieder lschen:
        final LgKunde_List kundenAlt = LgApplication.dummyKunde().alleSuchen();
        for(final LgKunde_List.Iterator i=kundenAlt.listIterator(); i.hasNext();){
			final LgKunde kunde = i.next();
            kunde.loeschen();
            assertTrue(kunde.isDeleted());
        }
        _mueller = _knabe = _skoric = null;

        //Jetzt mu die Tabelle wieder leer sein:
        final LgKunde_List kundenNeu = LgApplication.dummyKunde().alleSuchen();
		assertEquals(0, kundenNeu.size());

    }

    /**Erfasst einen Kunden mit diesem Namen und diesem Geburtsdatum persistent.
     * @param i_geburt Geburtsdatum in lokaler Schreibweise
    */
    private LgKunde _erfassen(final String i_name, final String i_geburt){
        final LgKunde kunde = new LgKunde();
        kunde.setName(i_name);
        kunde.setGeburt(LgDatum.fromString(i_geburt));
        kunde.erfassen();
        return kunde;
    }


}//class
